function [popListfinal, DDlist] = Sim_Res(thetm, time, N, I0, init, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
% USAGE: [popListfinal, DDlist] = Sim_Res(thetm, time, N, I0, init, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
%
% written by Laurinne Balstad, Contact Allison Shaw (ashaw@umn.edu) for assistance.
%
% Code to simulate the ecological dynamics for the matrix model.
% This produces two lists: one with the total population at time t0
% (to determine which classes are full/which thetas evolving)
% and one with the number of surviving adults who attempt to reproduce
% (for Density Dependence). These lists only contain one year, the year
% when invasion is attempted (T+T0).
%
% INPUTS:
%    thetam = current resident migration strategy
%    time = number of years to simulate for
%    N = the number of intensity classes/bins
%    I0 = the possible starting intensity classes --> row vector
%    init = the number of individuals in each class at start --> column vector
%    t0 = Time of migration decision
%    tR = Time between migrating and reproduction
%    tau = Half intensity time
%    a_r = Maximum resident mortality
%    c_r = Midpoint for resident mortality
%    a_m = Maximum migrant mortality
%    c_m = Midpoint for migrant mortality
%    D = “Slope” of mortality cost
%    f_r = Slope of birth cost, residents
%    f_m = Slope of birth cost, migrants
%    B_max = Maximum per capita fecundity
%    a1 = Density-dependent constant
%    a2 = Density-dependent constant
%
% OUTPUTS:
%    popListfinal = the number of individuals in each class at the
%      migration decision point -- this is used to determine which classes
%      have a theta that is under selective pressure versus just drifting
%    DDlist = number of surviving adults who are attempting to reproduce in


% ------------- MODEL PARAMETERS ---------------------------------------- %
thetr = 1 - thetm;
popList = []; % this will have the population by class for the final year (T0+T) at t_0
DDlist1 = []; % this will be used to calculate the DD
% ------------- MODEL PARAMETERS ---------------------------------------- %

for T = 1:time % need to go through all the years
        
    I0value = (2*I0 -1)/(2*N); % Convert the bins to Intensity Values
    Ir = (((I0value*tau)./(1-I0value))+t0)./(((I0value*tau)./(1-I0value))+t0+tau); % Now find the new intensities after time t0
    IR = ceil(Ir*N); % and then re-covert to bins
    

    % ------------- Matrix 1: Census --> DecisionPoint --------------------- %

    P1r = zeros(N); % Start with a matrix of zeros
    P1m = zeros(N);
    i = 1:length(IR);
    IRvalue = (2.*IR -1)./(2*N); % Figure out what the intensity actual is from the bin it's in
    Srvalue = 1 - (a_r./(1+exp(-D.*(IRvalue-c_r)))); % Use the intensity to find the survival
    Smvalue = 1 - (a_m./(1+exp(-D*(IRvalue-c_m))));
    Changes = floor(I0.*N -N)+IR; % This converts the old and new values to one value corresponding to the matrix index
    P1r(Changes) = Srvalue.*thetr(IR(i)); % Puts the survival probability in each spot
    P1m(Changes) = Smvalue.*thetm(IR(i));
    
    popthetas = (P1m+P1r)*init; % figure out what classes are full after just this point

    % ------------ Matrix 2: DecisionPoint --> Survival 2 ------------------ %

    Ir2 = (((IRvalue*tau)./(1-IRvalue))+tR)./(((IRvalue*tau)./(1-IRvalue))+tR+tau);
    IR2 = ceil(Ir2*N);

    % for residents: same structure
    
    P2r = zeros(N);
    IR2value = (2.*IR2 - 1)./(2*N);
    Sr2value = 1 - (a_r./(1+exp(-D.*(IR2value-c_r))));
    Changes2 = floor(IR.*N - N) + IR2;
    P2r(Changes2) = Sr2value;

    % Now consider migrants: they don't change class

    P2m = zeros(N);
    Changes3 = floor(IR.*N - N) + IR;
    P2m(Changes3) = Smvalue;

    popM2 = P2m*P1m; % figure out the population after this point: right before births
    popR2 = P2r*P2m;
    totpop = (popM2 + popR2)*init; % find the total population attempting to reproduce by class
    DD = a1*exp(-a2*sum(totpop)); % this is the density dependence for the year
    
    % ------------ Matrix 3: Survival 2 --> End of Year -------------------- %
    
    FecunRes = B_max - f_r*IR2value; % determine fecundity of residents
    FecunMig = B_max - f_m*IRvalue; % determine fecundity of migrants --> only for classes that are currently infected
    Changes5 = floor((IR2-1)*N + 1); % convert this to base N system so we know where to make changes 
    Changes6 = floor((IR-1)*N + 1); 
    P3rB1 = zeros(N); % create a matrix of zeros
    P3mB1 = zeros(N);
    P3rB1(Changes5) = FecunRes; % change the zeros matrix so it reflects only those that are infected: this also removes repeats
    P3mB1(Changes6) = FecunMig;
    
    % parents survive
    ASurm = diag(ones(1, N)); % overlapping
    ASurr = diag(ones(1, N));

    % Put it together
    P3m = DD*(P3mB1) + ASurm; % now rescale the births to only get survivors, also consider that no parents die
    P3r = DD*P3rB1 + ASurr;

    % ------------- Full transition: C(0) --> C(1) -------------------------- %

    Ptotm = P3m*P2m*P1m; % migrants year to year
    Ptotr = P3r*P2r*P1r; % residents year to year
    Ptot = Ptotm + Ptotr; % overall year to year
    init = Ptot*init; % recalculate population, use this one in next year
    I0 = find(init)'; % refind the infected classes, use this one in next year
    
end

popListfinal = popthetas; % this is the final year's survivors by class --> will tell nu us which thetas are 
                          % actually evolving as opposed to drift
DDlist = totpop; % this is the density dependence list for the final year when mutants invade: use this number
                 % to calculate the density dependence for the mutants

end