function [theta_mig] = findESS(N, I0, init, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
% USAGE: [theta_mig,theta_res_std,theta_res_mat,theta_res_std_mat,theta_hist] = findESS(N, I0, init, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
%
% written by Laurinne Balstad, Contact Allison Shaw (ashaw@umn.edu) for assistance.
%
% This searches for an Evolutionarily Stable Strategy (ESS)
%
% INPUTS:
%    N = the number of intensity classes/bins
%    I0 = the possible starting intensity classes --> row vector
%    init = the number of individuals in each class at start --> column vector
%    t0 = Time of migration decision
%    tR = Time between migrating and reproduction
%    tau = Half intensity time
%    a_r = Maximum resident mortality
%    c_r = Midpoint for resident mortality
%    a_m = Maximum migrant mortality
%    c_m = Midpoint for migrant mortality
%    D = “Slope” of mortality cost
%    f_r = Slope of birth cost, residents
%    f_m = Slope of birth cost, migrants
%    B_max = Maximum per capita fecundity
%    a1 = Density-dependent constant
%    a2 = Density-dependent constant
%
% OUTPUTS:
%    theta_mig = the ESS migration strategies

rng('shuffle') % randomize random number generator

%-----LOGISTICAL PARAMETERS-----------------------------------------------%
T0 = 100;      % generations of initial simulation
T  = 1;        % create one last year for invasion to occur

nmut = 2^N;    % number of different possible mutant strategy combinations
theta_mut_mat = dec2bin(2^N-1:-1:0)-'0'; % each row is a different mutant

count_tot_max = nmut; % number of invasions to attempt
count_res_max = 1; % number of times non-mutants must defend from mutants

theta_mig = ones(1, N); % start non-mutant population as fully migrating

%-----LOGISTICAL PARAMETERS-----------------------------------------------%

count_tot = 0;  % total invasion attempts
count_res = 0;  % successful resident defense

%------- ESS ANALYSIS ------------------------------------%

while (count_tot < count_tot_max && count_res < count_res_max)  % keep going unless (1) no ESS within limited attempts or (2) ESS reached

    count_tot = count_tot+1; % keep track of how many invasions have been attempted
    
    %-----SIMULATE RESIDENT POPULATION----------------------------------------%
    
    [totP, DDP] = Sim_Res(theta_mig, T0 + T, N, I0, init, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2);
    
    % this produces two lists: one with the total population at time t0
    % (to determine which classes are full/which thetas evolving)
    % and one with the number of surviving adults who attempt to reproduce
    % (for Density Dependence). These lists only contain one year, the year
    % when invasion is attempted (T+T0).
    
    %-------------------------------------------------------------------------%
    
    %-----INVASION-----------------------------------------%
    
    lyap_mcg = NaN(1,nmut); % this is a place to store the eigenvalues
    %theta_mut_mat = NaN(nmut+1,N); % this is a place to store the theta combinations

    for j = 1:nmut+1  % create nmut mutations following:
        if j==1; theta_mut = theta_mig; % (for first one, just invade with itself for a baseline)
        else
            theta_mut = theta_mut_mat(j-1,:); % otherwise pull from matrix of mutants
        end

        Jm = getMutTransMatrix(DDP, theta_mut, N, I0, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2);

        if sum(sum(1-isfinite(Jm)))~=0; error('jacobian error'); end % check the matrix didn't blow up
        
        lyap_mcg(j) = max(abs(eig(Jm))); % find the eigenvalue for each transition matrix
            
        clear Jm

    end  % theta-mut loop

    %-------------------------------------------------------------------------%
    
    % rounding error to go by
    ep = 0.0000001; 
    % find the index of all straegies that have the biggest growth rate, within the rounding error
    max_inv_all = find(abs(lyap_mcg - max(lyap_mcg))< ep);
    % find the index of the first invader (to see if it is the resident)
    max_inv = max_inv_all(1);
    
    if max_inv == 1           % resident wins
        count_res = 1;        % count resident wins --> because the resident was one of the maxes
    else                      % reset resident
        count_res = 0;        % reset resident counter
        theta_mig = theta_mut_mat(max_inv-1,:);
        % remember to shift by 1 because lyap_mcg is 1 longer than theta_mut_mat
        % (the first element of lyap_mcg is the resident strategy)
    end

end  % while loop for theta-res


%--------------------------------

% if last checked population is not viable, set ESS as NaN
if sum(totP) < 1
    theta_mig = NaN(1, N);
    disp('pop not viable')
    disp([sum(totP) max(totP)])
    
% otherwise, if no ESS reached, display error
elseif count_tot==count_tot_max
    error('no ESS found')

% otherwise output only the meaningful theta value as the ESS
else
    % look for the classes that have more than 1 individual at t0
    % (these are the only theta values that have meaning)
    theta_import = find(totP(:,1)>=1);
    
    % find the theta_import values for the resident
    theta_import_res = theta_mig(theta_import);
    
    % find the theta_import values for each of the maximum invaders from the last invasion attempt
    % note that the first value of max_inv_all will be 1 (the resident) - ignore this by using 2:end
    % note that max_inv_all runs to N+1 while theta_mut_max runs to N - correct by subtracting 1
    theta_import_inv = theta_mut_mat(max_inv_all(2:end)-1,theta_import);
    
    % check to see if any of the invaders are different from the resident in
    % any theta_import classes
    tmp = sum(sum(abs(theta_import_inv - repmat(theta_import_res,size(theta_import_inv,1),1))));
    if tmp>0;
        error('not a clean ESS - other strategies neutrally invade');
    end
    
    theta_final = NaN(1, N);
    theta_final(theta_import) = theta_mig(theta_import); % and only report those. Keep the others as Nan
    theta_mig = theta_final;
    
end

% if population not growing, output growth rate to check vs figures
ep2 = 0.001; % rounding error
if lyap_mcg(max_inv)<1-ep2
    disp([t0 tau lyap_mcg(max_inv)])
end


end

