function popMatrixMut = getMutTransMatrix(DDlist, thetmut, N, I0, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
% USAGE:  popMatrixMut = getMutTransMatrix(DDlist, thetmut, N, I0, t0, tR, tau, a_r, c_r, a_m, c_m, D, f_r, f_m, B_max, a1, a2)
%
% written by Laurinne Balstad, Contact Allison Shaw (ashaw@umn.edu) for assistance.
% 
% this produces the transition matrix that describes the growth of the
% mutant in a population of residents.
%
% INPUTS:
%    DDlist = number of surviving adults who are attempting to reproduce in
%    thetamut = current mutant migration strategy
%    N = the number of intensity classes/bins
%    I0 = the possible starting intensity classes --> row vector
%    t0 = Time of migration decision
%    tR = Time between migrating and reproduction
%    tau = Half intensity time
%    a_r = Maximum resident mortality
%    c_r = Midpoint for resident mortality
%    a_m = Maximum migrant mortality
%    c_m = Midpoint for migrant mortality
%    D = “Slope” of mortality cost
%    f_r = Slope of birth cost, residents
%    f_m = Slope of birth cost, migrants
%    B_max = Maximum per capita fecundity
%    a1 = Density-dependent constant
%    a2 = Density-dependent constant
%
% OUTPUT:
%    popMatrixMut = transition matrix that describes the growth of the
%      mutant in a population of residents

thetm = thetmut; % use these thetas for migrants
thetr = 1 - thetm; % and the complement for residents
        
I0value = (2*I0 -1)/(2*N); % approximate the inital intensity
Ir = (((I0value*tau)./(1-I0value))+t0)./(((I0value*tau)./(1-I0value))+t0+tau); % find new intensity
IR = ceil(Ir*N); % find new bin

% ------------- Matrix 1: Census --> DecisionPoint --------------------- %

% make the first transition matrix: need to find survivals and put
% values in correct spots based on changing class

P1r = zeros(N); % Start with a matrix of zeros
P1m = zeros(N);
i = 1:length(IR);
IRvalue = (2.*IR -1)./(2*N); % Figure out what the intensity actual is from the bin it's in
Srvalue = 1 - (a_r./(1+exp(-D.*(IRvalue-c_r)))); % Use the intensity to find the survival
Smvalue = 1 - (a_m./(1+exp(-D*(IRvalue-c_m))));
Changes = floor(I0.*N -N)+IR; % This converts the old and new values to one value corresponding to the matrix index
P1r(Changes) = Srvalue.*thetr(IR(i)); % Puts the survival probability in each spot
P1m(Changes) = Smvalue.*thetm(IR(i));

% ------------ Matrix 2: DecisionPoint --> Survival 2 ------------------ %

% Now we need to construct the second matrix. This will look different for
% residents and migrants, but the basic concept is the same as the first matrix

Ir2 = (((IRvalue*tau)./(1-IRvalue))+tR)./(((IRvalue*tau)./(1-IRvalue))+tR+tau); % find the resident's 2nd intensity
IR2 = ceil(Ir2*N); % and the associated bin

P2r = zeros(N); % same set up as the first matrix
IR2value = (2.*IR2 - 1)./(2*N);
Sr2value = 1 - (a_r./(1+exp(-D.*(IR2value-c_r))));
Changes2 = floor(IR.*N - N) + IR2;
P2r(Changes2) = Sr2value;

% Now consider migrants: they don't change class

P2m = zeros(N);
Changes3 = floor(IR.*N - N) + IR;
P2m(Changes3) = Smvalue;

% ------------ Matrix 3: Survival 2 --> End of Year -------------------- %

DD = a1*exp(-a2*sum(DDlist)); % get the density dependence from the SimResFinal

FecunRes = B_max - f_r*IR2value; % determine fecundity of residents
FecunMig = B_max - f_m*IRvalue; % determine fecundity of migrants --> only for classes that are currently infected
Changes5 = floor((IR2-1)*N + 1); % convert this to base N system so we know where to make changes 
Changes6 = floor((IR-1)*N + 1); 
P3rB1 = zeros(N); % create a matrix of zeros
P3mB1 = zeros(N);
P3rB1(Changes5) = FecunRes; % change the zeros matrix so it reflects only those that are infected: this also removes repeats
P3mB1(Changes6) = FecunMig;

% All the parents survive
ASurm = diag(ones(1, N)); % overlapping
ASurr = diag(ones(1, N));

% Put it together
P3m = DD*(P3mB1) + ASurm; % now rescale the births to only get survivors, also consider that no parents die
P3r = DD*P3rB1 + ASurr; 

% ------------- Full transition: C(0) --> C(1) -------------------------- %

Ptotm = P3m*P2m*P1m; % find migrants year to year
Ptotr = P3r*P2r*P1r; % find residents year to year
popMatrixMut = Ptotm + Ptotr; % add together to find whole population

end